--[[
	ForgetMeNot
	 	Long term cooldown reminder
	
	By: Daylight

	$Id$
	$Rev$
	$LastChangedBy$
	$Date$
]]--


-- Function Namespace
ForgetMeNot = {};

-- Constants
FORGETMENOT_VERSION = "1.02";

FORGETMENOT_REALMNAME = GetRealmName(); -- pseudo-constant
FORGETMENOT_PLAYERNAME = GetUnitName ("player"); -- pseudo-constant
FORGETMENOT_UPDATE_CYCLE = 60; -- seconds in which to recheck for finished cooldowns.

FORGETMENOT_DEBUG = false;  
-- FORGETMENOT_DEBUG_CHATFRAME = nil; 

-- Variables
ForgetMeNot.KhaosRegistered = nil; -- Khaos
ForgetMeNot.ConfigKhaos	= {}; -- khaos config save var

ForgetMeNot_SV		= {}; -- configuration save var (for no Khaos) -- saved variable
ForgetMeNot_SV_Data	= {}; -- data save var -- saved variable

ForgetMeNot.NextCheck	= GetTime() + 240; -- this variable will be set with a new value when the addon is fully loaded.
ForgetMeNot.StaticPopup = nil;

-- debugging stuff
ForgetMeNot.EventLog = {};


---------------------------------------------------------------------------------------------------------------------------

-----------------------------
-- CONFIGURATION FUNCTIONS --
-----------------------------

-------------------
-- SLASH HANDLER --
-------------------

--
--
--
function ForgetMeNot.SlashHandler(msg)
	
	msg = strlower(msg);

	slashcommand_reply = FORGETMENOT_SC_HELP_NOARGS; -- return this my default

	if (msg == "help") then  slashcommand_reply = FORGETMENOT_SC_HELPSC; end;
	if (msg == "report") then ForgetMeNot.ReportAll (); slashcommand_reply = ""; end ;
	if (msg == "cc" or msg == "clearcompleted") then ForgetMeNot.ClearCompleted (); slashcommand_reply = ""; end ;

	if (msg == "enable" or msg == "einschalten") then 
		slashcommand_reply=FORGETMENOT_MSG_ADDON_ENABLED;
		ForgetMeNot.ConfigSet ("Enabled",true,true);

		if (Khaos) then
			Khaos.setSetEnabled("ForgetMeNot", true);
			Khaos.refresh();
		end
	end
	if (msg == "disable" or msg == "ausschalten") then 
		slashcommand_reply=FORGETMENOT_MSG_ADDON_DISABLED;
		ForgetMeNot.ConfigSet ("Enabled",false,true);
		if (Khaos) then
			Khaos.setSetEnabled("ForgetMeNot", false);
			Khaos.refresh();
		end
	end

	if (msg == "autoreport on" or msg == "autoreport an") then
		slashcommand_reply=FORGETMENOT_MSG_REPORTONLOAD_ENABLED;
		ForgetMeNot.ConfigUpdate ("ReportOnLoad", "checked", true); 
	end

	if (msg == "autoreport off" or msg == "autoreport aus") then
		slashcommand_reply=FORGETMENOT_MSG_REPORTONLOAD_DISABLED;
		ForgetMeNot.ConfigUpdate ("ReportOnLoad", "checked", false); 
	end

	if (msg == "skipreport on" or msg == "skipreport an") then
		slashcommand_reply=FORGETMENOT_MSG_ONLYREPORTWHENFINISHED_ENABLED;
		ForgetMeNot.ConfigUpdate ("OnlyReportWhenFinished", "checked", true); 
	end

	if (msg == "skipreport off" or msg == "skipreport aus") then
		slashcommand_reply=FORGETMENOT_MSG_ONLYREPORTWHENFINISHED_DISABLED;
		ForgetMeNot.ConfigUpdate ("OnlyReportWhenFinished", "checked", false); 
	end

	if (msg == "status" or msg == "config") then
		slashcommand_reply=FORGETMENOT_SC_CONFIGREPORT..
		FORGETMENOT_SC_CONFIG_STATUS .."["..ForgetMeNot.ConvertBoolean(ForgetMeNot.ConfigGet("Enabled")).."]\n"..
		FORGETMENOT_SC_CONFIG_REPORT .."["..ForgetMeNot.ConvertBoolean(ForgetMeNot.ConfigGet("ReportOnLoad")).."]\n"..
		FORGETMENOT_SC_CONFIG_SKIPREPORT .."["..ForgetMeNot.ConvertBoolean(ForgetMeNot.ConfigGet("OnlyReportWhenFinished")).."]";
	end
	

	if (string.len (slashcommand_reply) >0 ) then ForgetMeNot.Print (nil,slashcommand_reply); end;
end



-----------
-- KHAOS --
-----------

function ForgetMeNot.ValidateConfig()
	if (not ForgetMeNot.ConfigGet ("Version",true)) then ForgetMeNot.ConfigSet ("Version","0.0.0",true); end; -- version info was not in SV yet	
  
	if (Khaos) then
		-- We have Khaos installed.
		if (not ForgetMeNot.KhaosRegistered) then ForgetMeNot.Print (nil,"[ForgetMeNot] ValidateConfig: Khaos detected but Addon not registered with Khaos. - This should not happen ;)"); end;
	

		-- version check
	
		if (ForgetMeNot.ConfigGet ("Version",true) < FORGETMENOT_VERSION) then
			ForgetMeNot.Print (nil,FORGETMENOT_MSG_CONFIGURATION_UPDATE);
			 ForgetMeNot.ConfigSet ("Version",FORGETMENOT_VERSION,true); -- set version to current version

			-- update config (KHAOS)
			-- do the update stuff
			
			-- set config vars SV (per character) but we dont use em for khaos
			if (ForgetMeNot.ConfigGet ("Enabled",true) == nil) then ForgetMeNot.ConfigSet ("Enabled",true,true); end;
			if (ForgetMeNot.ConfigGet ("ReportOnLoad",true) == nil) then ForgetMeNot.ConfigSet ("ReportOnLoad",false,true); end;
			if (ForgetMeNot.ConfigGet ("OnlyReportWhenFinished",true) == nil) then ForgetMeNot.ConfigSet ("OnlyReportWhenFinished",true,true); end;
		end
	else
		-- No Khaos installed.
		if (ForgetMeNot.ConfigGet ("Version",true) < FORGETMENOT_VERSION) then
			ForgetMeNot.Print (nil,FORGETMENOT_MSG_CONFIGURATION_UPDATE);
			 ForgetMeNot.ConfigSet ("Version",FORGETMENOT_VERSION,true); -- set version to current version

			-- set config vars SV (per character)
			if (ForgetMeNot.ConfigGet ("Enabled") == nil ) then ForgetMeNot.ConfigSet ("Enabled",true); end;
			if (ForgetMeNot.ConfigGet ("ReportOnLoad") == nil ) then ForgetMeNot.ConfigSet ("ReportOnLoad",false); end;
			if (ForgetMeNot.ConfigGet ("OnlyReportWhenFinished",true) == nil) then ForgetMeNot.ConfigSet ("OnlyReportWhenFinished",true); end;
		end
	end
end

function ForgetMeNot.ValidateData()
	if (not ForgetMeNot_SV_Data["version"]) then ForgetMeNot_SV_Data["version"] = "0.0.0"; end; -- version info was not in SV yet	
  

	if (ForgetMeNot_SV_Data["version"] < FORGETMENOT_VERSION) then
		 ForgetMeNot_SV_Data["version"] = FORGETMENOT_VERSION; -- set version to current version

		 -- if we need updates to the datafile, we need to process data for every character now [data][player - realm]
	end

	-- following needs to be checked always


	-- data exsists
	if (not ForgetMeNot_SV_Data["data"]) then ForgetMeNot_SV_Data["data"] = {}; end;

	-- data for this player exsists ?

	if (not ForgetMeNot_SV_Data["data"][FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME]) then 
		-- ForgetMeNot.Print (99,"Creating data entry for "..FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME);
		ForgetMeNot_SV_Data["data"][FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME] = {}; 
	end
	
end


-- registers addon with khaos
function ForgetMeNot.RegisterKhaos ()
	local optionset =
		{
			id="ForgetMeNot";
			text=FORGETMENOT_CONFIG_HEADER;
			helptext=FORGETMENOT_CONFIG_HEADER_INFO;
			difficulty=1;
			callback = function(state) 
				if ( state==true ) then 
					ForgetMeNot.ConfigSet("Enabled",true); 
					-- more initialisation
				else ForgetMeNot.ConfigSet("Enabled",false);end; 

				ForgetMeNotFrame.loadedKhaosSet=true; 
				end;
			feedback = function(state)
				if ( state==true ) then return (FORGETMENOT_MSG_ADDON_ENABLED); else return (FORGETMENOT_MSG_ADDON_DISABLED); end; end;
			options = {
					{
						id="Header";
						text=FORGETMENOT_CONFIG_HEADER;
						helptext=FORGETMENOT_CONFIG_HEADER_INFO;
						type=K_HEADER;
						difficulty=1;
					};

					{
						id="ReportOnLoad";
						text=FORGETMENOT_REPORTONLOAD; 
						helptext=FORGETMENOT_REPORTONLOAD_INFO;
				                type = K_TEXT;
                  				value = true;
						check = true;
						callback = function(state) ForgetMeNot.ConfigSet("ReportOnLoad",state.checked); end;
				                feedback = function(state) 
			                        if ( state.checked ) then
							return FORGETMENOT_MSG_REPORTONLOAD_ENABLED;
							else
								return FORGETMENOT_REPORTONLOAD_DISABLED;
							end
				                end;
						default = {
							checked = false;
						};
						disabled = {
							checked = false;
						};
					};

					{
						id="OnlyReportWhenFinished";
						text=FORGETMENOT_ONLYREPORTWHENFINISHED; 
						helptext=FORGETMENOT_ONLYREPORTWHENFINISHED_INFO;
				                type = K_TEXT;
                  				value = true;
						check = true;
						callback = function(state) ForgetMeNot.ConfigSet("OnlyReportWhenFinished",state.checked); end;
				                feedback = function(state) 
			                        if ( state.checked ) then
							return FORGETMENOT_MSG_ONLYREPORTWHENFINISHED_ENABLED;
							else
								return FORGETMENOT_ONLYREPORTWHENFINISHED_DISABLED;
							end
				                end;
						default = {
							checked = true;
						};
						disabled = {
							checked = false;
						};
						dependencies = {
							["ReportOnLoad"] = { checked=true }; 
						};
					};



              -- };


				
				  }; -- option end
		}; -- optionset end

	if (Khaos.registerOptionSet("other", optionset)) then 
		ForgetMeNot.KhaosRegistered = true;
	else
		ForgetMeNot.Print (0,"RegisterKhaos: Couldn'r register with Khaos - this should not happen ;)");
	end
end

function ForgetMeNot.SecondsToHumanReadable (Seconds)
	local TimeString = "";	

	local Days = math.floor(Seconds / 86400);
	if (Days >= 1) then  Seconds = Seconds - (Days*86400); end;
	TimeString=TimeString..Days..":";

	local Hours = math.floor (Seconds / 3600);
	if (Hours >=1) then Seconds = Seconds - (Hours*3600); end;
	TimeString=TimeString..string.format('%.2i:',Hours);	

	local Minutes = math.floor (Seconds / 60);
	if (Minutes >=1) then Seconds = Seconds - (Minutes*60); end;
	TimeString=TimeString..string.format('%.2i:',Minutes);	

	TimeString=TimeString..string.format('%.2i',Seconds);	
	
	return (TimeString);

end


---------------------------------------------------------------------------------------------------------------------------

--------------------
-- MISC FUNCTIONS --
--------------------

---------------------------------------------------------------------------------------------------------------------------

-------------
-- GENERAL --
-------------

--
--
--

function ForgetMeNot.Print (level,msg)

	if (level == nil) then
		-- unfiltered print, always
		DEFAULT_CHAT_FRAME:AddMessage (msg);
		if (FORGETMENOT_DEBUG_CHATFRAME) then 
			-- if exists also print to debug chatframe
			FORGETMENOT_DEBUG_CHATFRAME:AddMessage (msg);
		end
		return;
	end

	if (level ~= nil) then
		if (FORGETMENOT_DEBUG == true) then
			-- debug
			DEFAULT_CHAT_FRAME:AddMessage ("[ForgetMeNot Debug] "..msg,0.5,0.5,1);
		end
		if (FORGETMENOT_DEBUG_CHATFRAME) then 
			-- if exists also print to debug chatframe
			FORGETMENOT_DEBUG_CHATFRAME:AddMessage (msg);
		end
		return;
	end
end


-- DEBUG STUFF BEGIN

--
function ForgetMeNot.GetDebugChatframe (TabName)
	for i=1,NUM_CHAT_WINDOWS do
		if (GetChatWindowInfo (i) == TabName) then return (getglobal("ChatFrame"..i)); end;
	end
	return (nil);
end


--
function ForgetMeNot.LogEvent (func, event)
	table.insert (ForgetMeNot.EventLog,{func,event});
end
--

-- DEBUG STUFF END

function ForgetMeNot.ConfigSet (stor_var,value,ignoreKhaos)
-- sets configuration variable (stor_var) to value (value)
	if (Khaos and not ignoreKhaos) then
		ForgetMeNot.ConfigKhaos[stor_var]=value; -- for Khaos
	else
		-- if the profile doesnt exist, create one
		if (not ForgetMeNot_SV[FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME]) then 
			ForgetMeNot_SV[FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME] = {};
		end
		ForgetMeNot_SV[FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME][stor_var]=value; -- for WTF
	end
end

-- returns configuration variable (stor_var)
function ForgetMeNot.ConfigGet (stor_var, ignoreKhaos)
	-- for Khaos
	if (Khaos and not ignoreKhaos) then
		return (ForgetMeNot.ConfigKhaos[stor_var]);
	else
		-- if the profile doesnt exist, return nil
		if (not ForgetMeNot_SV[FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME]) then return (nil); end;

		return (ForgetMeNot_SV[FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME][stor_var]);
	end
end

-- updates configuration (be called from /commands)
function ForgetMeNot.ConfigUpdate (key, parameter, value)
	if (Khaos) then
		Khaos.setSetKeyParameter("ForgetMeNot",key,parameter,value);
		ForgetMeNot.ConfigSet (key,value);
		Khaos.refresh();
	else
		ForgetMeNot.ConfigSet (key,value);
	end
end

--

--
function ForgetMeNot.ConvertBoolean (value)
	if (value==true) then return (FORGETMENOT_MSG_ON); end;
	if (value==false) then return (FORGETMENOT_MSG_OFF); end;
	
	return (value);
end

function ForgetMeNot.PrintStatus ()
	-- Welcome message
	if (FORGETMENOT_DEBUG == true) then
		ForgetMeNot.Print (nil,"ForgetMeNot V"..FORGETMENOT_VERSION.." initialized.");	
	end
end


function ForgetMeNot.ReportOnLoad ()
	-- this is to be called on load
	-- this prints all cooldowns those finished and those who are not

	if (ForgetMeNot.ConfigGet("ReportOnLoad") == true and ForgetMeNot.ConfigGet("Enabled") == true) then
		ForgetMeNot.ReportAll (true);
	end
end

function ForgetMeNot.ReportAll (OnLoad)
	-- this prints all cooldowns those finished and those who are not
	-- this is basically the same as ReportOnLoad
	local Report = FORGETMENOT_MSG_COOLDOWNREPORT_HEADER.."\n\n";
	local ReportEmpty = true;
	local CooldownsCompleted = nil;
	local FinishedCooldowns = false;
	for PlayerName,PlayerData in pairs (ForgetMeNot_SV_Data.data) do
		local CooldownReportEntry = "";
		NewPlayerData = {};
		for SpellName,CooldownFinished in pairs (PlayerData) do
			if (CooldownFinished < time()) then
				-- this cooldown is finished
				CooldownReportEntry = CooldownReportEntry .. "    " .. SpellName .. " - " .. FORGETMENOT_MSG_CDFINISHED.."\n";
				NewPlayerData[SpellName]=0; -- ,mark as finished so it wont be printed again
				CooldownsCompleted = true;
				FinishedCooldowns = true;
			else
				-- this cooldown is not finished
				CooldownReportEntry = CooldownReportEntry .. "    "  .. SpellName .. " - " .. FORGETMENOT_MSG_CDNOTFINISHED..ForgetMeNot.SecondsToHumanReadable(CooldownFinished-time())..")\n";
				NewPlayerData[SpellName]=CooldownFinished;
			end


		end
		ForgetMeNot_SV_Data.data[PlayerName]=NewPlayerData;

		if (#CooldownReportEntry ~= 0) then
			-- "print" the block if theres something to print
			Report = Report .. "["..PlayerName.."]\n" .. CooldownReportEntry.."\n";
			ReportEmpty = false;
		end
		
	end
	
	Report = Report .. FORGETMENOT_MSG_COOLDOWNREPORT_FOOTER;


	if (ForgetMeNot.ConfigGet("ReportOnLoad") == true and ForgetMeNot.ConfigGet("OnlyReportWhenFinished") == true and FinishedCooldowns == false and OnLoad) then
		-- nothing to report
		ForgetMeNot.Print (nil,FORGETMENOT_MSG_NOTINGFINISHED_NOREPORT);
		return;
	end
	
	if (ReportEmpty == false) then
		ForgetMeNot.ReportFrame.Box:SetText(Report)
		ForgetMeNot.ReportFrame.Scroll:UpdateScrollChildRect();
		ForgetMeNot.ReportFrame:Show();
		if (CooldownsCompleted) then
			ForgetMeNot.ReportFrame.ClearCompleted:Show();
			ForgetMeNot.ReportFrame.Mesg:Show();
		end
	else
		-- nothing to report
		ForgetMeNot.Print (nil,FORGETMENOT_MSG_NOREPORT);
	end
end

--
function ForgetMeNot.ReportOnUpdate ()
	-- this is to be called on update
	-- this prints if a cooldown just finished
	local CooldownReport = "";
	for PlayerName,PlayerData in pairs (ForgetMeNot_SV_Data.data) do
		local CooldownReportEntry = "";
		NewPlayerData = {};
		for SpellName,CooldownFinished in pairs (PlayerData) do

			if (CooldownFinished > 0 and CooldownFinished < time()) then
				-- this cooldown is finished
				ForgetMeNot.Print (nil,"\n"..FORGETMENOT_MSG_COOLDOWNREPORT_HEADER.."\n["..PlayerName.."] - ".. SpellName .. " - " .. FORGETMENOT_MSG_CDFINISHED.."\n"..FORGETMENOT_MSG_COOLDOWNREPORT_FOOTER);
				NewPlayerData[SpellName]=0; -- ,mark as finished so it wont be printed again
			else
				-- this cooldown is not finished
				NewPlayerData[SpellName]=CooldownFinished;
			end
		end
		ForgetMeNot_SV_Data.data[PlayerName]=NewPlayerData;
	end
end


function ForgetMeNot.ClearCompleted ()
	-- clears all finshed cooldowns
	for PlayerName,PlayerData in pairs (ForgetMeNot_SV_Data.data) do
		local CooldownReportEntry = "";
		NewPlayerData = {};
		for SpellName,CooldownFinished in pairs (PlayerData) do

			if (CooldownFinished < time()) then
				-- this cooldown is finished
			else
				-- this cooldown is not finished
				NewPlayerData[SpellName]=CooldownFinished;
			end


		end
		ForgetMeNot_SV_Data.data[PlayerName]=NewPlayerData;
	end
	

	ForgetMeNot.Print (nil,FORGETMENOT_MSG_CLEAREDCOMPLETED);
end

function ForgetMeNot.ReportFrameClearCompleted ()
	ForgetMeNot.ClearCompleted ();
 	ForgetMeNot.ReportFrame.ClearCompleted:Hide();
	ForgetMeNot.ReportFrame.Mesg:Hide();
	ForgetMeNot.ReportFrame:Hide();
	ForgetMeNot.ReportAll ();
end

function ForgetMeNot.ReportFrameHide()
	ForgetMeNot.ReportFrame.ClearCompleted:Hide();
	ForgetMeNot.ReportFrame.Mesg:Hide();
	ForgetMeNot.ReportFrame:Hide();
end

function ForgetMeNot.PopupDone (action,spellname)
	ForgetMeNot.StaticPopup = nil;
	if (action == "USE") then
		ForgetMeNot.AddCooldown (spellname);
	else
		ForgetMeNot.Print (99,"craft...");
	end

end


function ForgetMeNot.ItemCooldownDialog (spellname)
	-- now check if we have a staticPopup open already

	if (ForgetMeNot.StaticPopup) then
		-- has to be a crafted item, dont do anything
		ForgetMeNot.Print (99,"skipping Popup, must be crafted a crafted item");
		return;
	end
	
	StaticPopupDialogs["FORGETMENOT_POPUP"] = {
		text = FORGETMENOT_MSG_POPUP1..spellname..FORGETMENOT_MSG_POPUP2,
		button1 = FORGETMENOT_BUTTON_POPUP_USE,
		button2 = FORGETMENOT_BUTTON_POPUP_CRAFT,
		OnAccept = function()  ForgetMeNot.PopupDone ("USE",spellname); end,
		OnCancel = function()  ForgetMeNot.PopupDone ("CRAFT",spellname); end,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1
	}
	ForgetMeNot.StaticPopup = true;
	StaticPopup_Show ("FORGETMENOT_POPUP");
end

function ForgetMeNot.AddCooldown (spellname)
	local spelldata = FORGETMENOT_SPELLDATA[spellname]
			
	if (ForgetMeNot_SV_Data["data"][FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME][spellname]) then
		if (ForgetMeNot_SV_Data["data"][FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME][spellname] > 0) then
			ForgetMeNot.Print (nil,"[ForgetMeNot] "..spellname..FORGETMENOT_MSG_BUGREPORT);
		end
	end
	ForgetMeNot_SV_Data["data"][FORGETMENOT_PLAYERNAME.." - "..FORGETMENOT_REALMNAME][spellname] = time()+spelldata.cooldown;
	ForgetMeNot.Print (nil,"[ForgetMeNot] "..spellname..FORGETMENOT_MSG_PROCESS..ForgetMeNot.SecondsToHumanReadable(spelldata.cooldown)..".");
end


-- /script ForgetMeNot.CheckCooldown("Salt Shaker");
function ForgetMeNot.CheckCooldown (spellname)
	local spelldata = FORGETMENOT_SPELLDATA[spellname]

	if (spelldata) then
		if (spelldata.craftable) then
			ForgetMeNot.ItemCooldownDialog (spellname)
			return;
		end
		ForgetMeNot.AddCooldown (spellname);
	end
	-- this not a spell we process
	return;
end



---------------------------------------------------------------------------------------------------------------------------

-------------------------
-- INITIALISATION PART --
-------------------------

function ForgetMeNot.Initialize ()

	-- set debug chatframe if debug is set
	if (FORGETMENOT_DEBUG == true) then
		FORGETMENOT_DEBUG_CHATFRAME = ForgetMeNot.GetDebugChatframe ("ForgetMeNot-Debug"); 
	end

	ForgetMeNot.ValidateConfig();
	ForgetMeNot.ValidateData();
	
	-- initialisation

	ForgetMeNotFrame.isInitialized = true;
	
	-- debug code
	ForgetMeNot.EventLog = {}

	-- REPORT FRAME STUFF

	ForgetMeNot.ReportFrame = CreateFrame("Frame", "", UIParent)
	ForgetMeNot.ReportFrame:Hide()
	ForgetMeNot.ReportFrame:SetPoint("CENTER", "UIParent", "CENTER")
	ForgetMeNot.ReportFrame:SetFrameStrata("TOOLTIP")
	ForgetMeNot.ReportFrame:SetHeight(280)
	ForgetMeNot.ReportFrame:SetWidth(650)
	ForgetMeNot.ReportFrame:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = { left = 9, right = 9, top = 9, bottom = 9 }
	})
	ForgetMeNot.ReportFrame:SetBackdropColor(0,0,0, 0.8)
	ForgetMeNot.ReportFrame:SetScript("OnShow", ForgetMeNot.ReportFrameShow)

	ForgetMeNot.ReportFrame.Done = CreateFrame("Button", "", ForgetMeNot.ReportFrame, "OptionsButtonTemplate")
	ForgetMeNot.ReportFrame.Done:SetText(FORGETMENOT_BUTTON_CLOSE)
	ForgetMeNot.ReportFrame.Done:SetPoint("BOTTOMRIGHT", ForgetMeNot.ReportFrame, "BOTTOMRIGHT", -10, 10)
	ForgetMeNot.ReportFrame.Done:SetScript("OnClick", ForgetMeNot.ReportFrameHide)

	ForgetMeNot.ReportFrame.ClearCompleted = CreateFrame("Button", "", ForgetMeNot.ReportFrame, "OptionsButtonTemplate")
	ForgetMeNot.ReportFrame.ClearCompleted:SetText(FORGETMENOT_BUTTON_CLEAR)
	ForgetMeNot.ReportFrame.ClearCompleted:SetPoint("BOTTOMRIGHT", ForgetMeNot.ReportFrame.Done, "BOTTOMLEFT", -5, 0)
	ForgetMeNot.ReportFrame.ClearCompleted:SetScript("OnClick", ForgetMeNot.ReportFrameClearCompleted)
	ForgetMeNot.ReportFrame.ClearCompleted:Hide()

	ForgetMeNot.ReportFrame.Mesg = ForgetMeNot.ReportFrame:CreateFontString("", "OVERLAY", "GameFontNormal")
	ForgetMeNot.ReportFrame.Mesg:SetJustifyH("LEFT")
	ForgetMeNot.ReportFrame.Mesg:SetPoint("TOPRIGHT", ForgetMeNot.ReportFrame.Done, "TOPLEFT", -10, 0)
	ForgetMeNot.ReportFrame.Mesg:SetPoint("LEFT", ForgetMeNot.ReportFrame, "LEFT", 15, 0)
	ForgetMeNot.ReportFrame.Mesg:SetHeight(20)
	ForgetMeNot.ReportFrame.Mesg:SetText(FORGETMENOT_TEXT_CLEAR)
	ForgetMeNot.ReportFrame.Mesg:Hide()

	ForgetMeNot.ReportFrame.Scroll = CreateFrame("ScrollFrame", "ForgetMeNotInputScroll", ForgetMeNot.ReportFrame, "UIPanelScrollFrameTemplate")
	ForgetMeNot.ReportFrame.Scroll:SetPoint("TOPLEFT", ForgetMeNot.ReportFrame, "TOPLEFT", 20, -20)
	ForgetMeNot.ReportFrame.Scroll:SetPoint("RIGHT", ForgetMeNot.ReportFrame, "RIGHT", -30, 0)
	ForgetMeNot.ReportFrame.Scroll:SetPoint("BOTTOM", ForgetMeNot.ReportFrame.Done, "TOP", 0, 10)

	ForgetMeNot.ReportFrame.Box = CreateFrame("EditBox", "ForgetMeNotEditBox", ForgetMeNot.ReportFrame.Scroll)
	ForgetMeNot.ReportFrame.Box:SetWidth(600)
	ForgetMeNot.ReportFrame.Box:SetHeight(85)
	ForgetMeNot.ReportFrame.Box:SetMultiLine(true)
	ForgetMeNot.ReportFrame.Box:SetAutoFocus(false)
	ForgetMeNot.ReportFrame.Box:SetFontObject(GameFontHighlight)
	ForgetMeNot.ReportFrame.Box:SetScript("OnEditFocusGained", ForgetMeNot.ReportFrameFocusGained)
	ForgetMeNot.ReportFrame.Box:EnableKeyboard(false)

	ForgetMeNot.ReportFrame.Scroll:SetScrollChild(ForgetMeNot.ReportFrame.Box)
end


---------------------------------------------------------------------------------------------------------------------------

--------------------
-- CORE FUNCTIONS --
--------------------

---------------------------------------------------------------------------------------------------------------------------

---------------------
-- ON-LOAD HANDLER --
---------------------
--
function ForgetMeNot.OnLoad ()
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_LOGIN");
	if (FORGETMENOT_LOCALE) then
		this:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");

		-- initialize slash handler  
		SlashCmdList["FORGETMENOT"] = ForgetMeNot.SlashHandler;
		SLASH_FORGETMENOT1 = "/ForgetMeNot";
		SLASH_FORGETMENOT2 = "/fmn";
	end
end

---------------------------------------------------------------------------------------------------------------------------
-----------------------
-- ON-UPDATE HANDLER --
-----------------------
function ForgetMeNot.OnUpdate()

	-- PRINT STATUS MESSAGE PART 

	if (not ForgetMeNotFrame.isReady) then
		if (Khaos) then
			if (ForgetMeNotFrame.loadedKhaosSet == true) then
				-- our Khaos set was iniitialized
				if (FORGETMENOT_LOCALE) then
					ForgetMeNot.PrintStatus ();
					ForgetMeNotFrame.isReady = true;
					ForgetMeNot.NextCheck = GetTime() + FORGETMENOT_UPDATE_CYCLE;
					ForgetMeNot.ReportOnLoad ();
				else
					
					DEFAULT_CHAT_FRAME:AddMessage (FORGETMENOT_MSG_INCOMPATIBLE,1,0,0);
					ForgetMeNotFrame.isReady = true;
				end
			end
		else
			if (ForgetMeNotFrame.variablesLoaded == true) then
				if (FORGETMENOT_LOCALE) then
					ForgetMeNot.PrintStatus ();
					ForgetMeNotFrame.isReady = true;
					ForgetMeNot.NextCheck = GetTime() + FORGETMENOT_UPDATE_CYCLE;
					ForgetMeNot.ReportOnLoad ();
				else
					DEFAULT_CHAT_FRAME:AddMessage (FORGETMENOT_MSG_INCOMPATIBLE,1,0,0);
					ForgetMeNotFrame.isReady = true;
				end
			end
		end
	end

	-- DEBUG CODE --
	if (table.getn(ForgetMeNot.EventLog) >500) then
		for i=1, 100, 1 do table.remove (ForgetMeNot.EventLog,1); end;
	end

	-- DEBUG CODE END


	if (ForgetMeNot.ConfigGet ("Enabled") and FORGETMENOT_LOCALE ) then
		-----------------------------------
		-- ONUPDATE - WHEN ADDON ENABLED --
		-----------------------------------

		if (ForgetMeNot.NextCheck < GetTime () ) then
			ForgetMeNot.ReportOnUpdate ();
			ForgetMeNot.NextCheck = GetTime() + FORGETMENOT_UPDATE_CYCLE;
		end
	end 
	
	-- next thing to do on-update 
end

---------------------------------------------------------------------------------------------------------------------------


-----------------------
-- ON-EVENT HANDLERS --
-----------------------
--
function ForgetMeNot.OnEvent()
	
	-----------------------------
	-- EVENT: VARIABLES_LOADED --
	-----------------------------
	--
	if (event == "VARIABLES_LOADED") then
		ForgetMeNot.EventLog.variablesLoaded = true;

		-- If Khaos is present, register the addon
		if (Khaos) then  ForgetMeNot.RegisterKhaos(); end
	end

	-------------------------
	-- EVENT: PLAYER_LOGIN --
	-------------------------
	--

	if (event == "PLAYER_LOGIN") then
		ForgetMeNot.Initialize (); 
	end

	
	-----------------------------------------
	-- EVENTS PROCESSED WHEN ADDON IS ENABLED
	-----------------------------------------
	if (ForgetMeNot.ConfigGet ("Enabled")) then
		-- is Enabled

		-------------------------------------
		-- EVENT: UNIT_SPELLCAST_SUCCEEDED --
		-------------------------------------
		--
		if (event == "UNIT_SPELLCAST_SUCCEEDED") then
			--  arg1 -  Unit casting the spell 
			--  arg2 -  Spell name 
			--  arg3 -  Spell rank
			
			
			if (arg1 ~= "player") then 
				return; -- we're not interested in other players spellcasts
			end

			-- do stuff
			local spellname = arg2;
			ForgetMeNot.LogEvent ("Spell cast success",spellname);
			ForgetMeNot.CheckCooldown (spellname);

		end

	
	end -- isActive

	-- next event
end		

---------------------------------------------------------------------------------------------------------------------------
